@@ -14,18 +14,17 @@ test_that("make_mc(g)ev work properly", {
 
 
     ## examples from mc_factorize.Rd
-mat2 <- make_mcmatrix(eigval = c(1), co = cbind(c(1,1,1,1), c(0,1,0,0)), dim = 4, len.block = c(2))
-mat2
-eigen(mat2)
+    mat2 <- make_mcmatrix(eigval = c(1), co = cbind(c(1,1,1,1), c(0,1,0,0)),
+                          dim = 4, len.block = c(2))
+    mat2
+    eigen(mat2)
     mc_leftc(mat2, mo = 4, mo.col = 2)
-    expect_error(mc_leftc(mat2, mo = 4),
-                 "singular matrix 'a' in solve")
+    expect_error(mc_leftc(mat2, mo = 4), "singular matrix 'a' in solve")
     
-mCompanion(mat2)
-mCompanion(mat2, mo=4, mo.col=2)
-mc_leftc(mCompanion(mat2), mo = 4, mo.col = 2)
-mc_eigen(mCompanion(mat2), mo = 4, mo.col = 2)
-mc_eigen(mCompanion(mat2, mo=4, mo.col=2), mo = 4, mo.col = 2)
+    mCompanion(mat2)
+    mCompanion(mat2, mo = 4, mo.col = 2)
+    mc_leftc(mCompanion(mat2), mo = 4, mo.col = 2)
+    mc_eigen(mCompanion(mat2), mo = 4, mo.col = 2)
+    mc_eigen(mCompanion(mat2, mo = 4, mo.col = 2), mo = 4, mo.col = 2)
 
-    
 })
